unit Install;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, FileCtrl, ExtCtrls, Gauges;

{$DEFINE Shareware}

Const
  CpNb = 21;
  ExNb = 18;

type
  TfmInstall = class(TForm)
    btCancel: TSpeedButton;
    btPrevious: TSpeedButton;
    btNext: TSpeedButton;                    
    btInstall: TSpeedButton;
    Notebook1: TNotebook;
    mAccueil: TMemo;
    Bevel1: TBevel;
    Label2: TLabel;
    btChooseRepert: TSpeedButton;
    Label3: TLabel;
    Gauge1: TGauge;
    Label4: TLabel;
    lbOperation: TLabel;
    Label6: TLabel;
    Label1: TLabel;
    btChooseRepertExample: TSpeedButton;
    edRepert: TEdit;
    edFree: TEdit;
    edNeeded: TEdit;
    edRepertExamples: TEdit;
    mBye: TMemo;
    btOk: TSpeedButton;
    Label7: TLabel;
    Label8: TLabel;
    edSociete: TEdit;
    edNom: TEdit;
    Label9: TLabel;
    Label10: TLabel;
    edRegistrationID: TEdit;
    lbVersion: TLabel;
    lbReg: TLabel;
    rg16or32: TRadioGroup;
    Bevel3: TBevel;
    Label11: TLabel;
    btChooseRepertSource: TSpeedButton;
    edSource: TEdit;
    Label12: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure btInstallClick(Sender: TObject);
    procedure btOkClick(Sender: TObject);
    procedure btChooseRepertClick(Sender: TObject);
    procedure edRepertChange(Sender: TObject);
    procedure btPreviousClick(Sender: TObject);
    procedure btNextClick(Sender: TObject);
    procedure Notebook1PageChanged(Sender: TObject);
    procedure btCancelClick(Sender: TObject);
    procedure btChooseRepertSourceClick(Sender: TObject);
  private
    { Dclarations private }
    Procedure CopyFiles;
    Procedure SetIndicators(V: Boolean);
    Procedure Indique(Mess : String);
    Procedure Progress(P : Integer);
    Function Run: Boolean;
  public
    { Dclarations public }
    Source, Destination, DestEx : String;
    Dispo, Needed : Double;
    OpEC, OpTot : Integer;
  end;

{ Utilities }
Procedure drExcept(Const Mess : String);
Function YesNo(Const Mess : String) : Boolean;
Function DeleteFile(pFile : String; Var pErr: String): Boolean;
Function CopyFile(pSource, pDest : String; Var pErr: String): Boolean;
Function CreateFile(pDest : String; Var pErr: String): Boolean;

var
  fmInstall: TfmInstall;

implementation

Uses IniFiles, LZExpand;

{$R *.DFM}

Function StrEspace(Esp: Double): String;
Var
  S : String;
Begin
  S := FormatFloat('### ### ### ###', Esp);
  While S[1] = ' ' Do Delete(S, 1, 1);
  Result := S;
End;

procedure TfmInstall.btPreviousClick(Sender: TObject);
begin
  Notebook1.PageIndex := Notebook1.PageIndex - 1;
end;

procedure TfmInstall.btNextClick(Sender: TObject);
begin
  If (Notebook1.ActivePage = 'References') Then
  Begin
    If (edNom.Text = '') And (edSociete.Text = '') Then
      drExcept('Please Enter User or Company Name');
    If edRegistrationID.Text = '' Then
      drExcept('Please Enter Registration ID');
  End;
  Notebook1.PageIndex := Notebook1.PageIndex + 1;
end;

procedure TfmInstall.FormCreate(Sender: TObject);
Var S : String;
begin
  Needed := 450000;
  edNeeded.Text := StrEspace(Needed);
  Notebook1.PageIndex := 0;
  SetIndicators(False);
  Source := ParamStr(0);
  Source := ExtractFilePath(Source);
  Delete(Source, Length(Source), 1);      { Last \ }
  S := Source;
  edRepert.Text := S;
  edRepertExamples.Text := S;
  edSource.Text := S;
  OpTot := CpNb + ExNb + 3;
  edRepertChange(Sender);
{$IFDEF Shareware}
  edRegistrationID.Text := 'Shareware';
  edRegistrationID.Enabled := False;
{$ELSE}
  lbVersion.Caption := 'Registered Version';
{$ENDIF}
  Gauge1.MaxValue := OpTot;
end;

procedure TfmInstall.btCancelClick(Sender: TObject);
begin
  Close;
end;

Procedure TfmInstall.SetIndicators(V: Boolean);
Begin
  Gauge1.Visible := V;
  lbOperation.Visible := V;
  Application.ProcessMessages;
End;

Procedure TfmInstall.Progress(P : Integer);
Begin
  Gauge1.Progress := P;
  Application.ProcessMessages;
End;

Procedure TfmInstall.Indique(Mess : String);
Begin
  lbOperation.Caption := Mess;
  Progress(OpEC);
  Inc(OpEc);
  Application.ProcessMessages;
End;

Procedure TfmInstall.CopyFiles;
Var
  Src, Dst, Err, Ext : String;
  CpF :  Array[1..CpNb] Of String[12];
  ExF :  Array[1..ExNb] Of String[12];
  I : Integer;

  Procedure FicCopy(Fic: String);
  Begin
    Indique('Copying ' + Fic);
    If Src + Fic <> Dst + Fic Then
      If Not CopyFile(Src + Fic, Dst + Fic, Err) Then
        Raise Exception.Create(Err);
  End;
  Procedure FicDelete(Fic: String);
  Begin
    If Not DeleteFile(Src + Fic, Err) Then
      Raise Exception.Create(Err);
  End;
Begin
  Src := Source + '\';
  Dst := Destination + '\';
{$IFDEF Shareware}
  Ext := '.DCU';
{$ELSE}
  Ext := '.PAS';
{$ENDIF}

  { Fill Arrays with File Names }
  { Component }
  CpF[1] := 'DRFRMAST' + Ext;
  CpF[2] := 'DRFRMAST.DFM';
  CpF[3] := 'DRJOIN' + Ext;
  CpF[4] := 'DRJOIN.DFM';
  CpF[5] := 'DRNEWTBL' + Ext;
  CpF[6] := 'DRNEWTBL.DFM';
  CpF[7] := 'DRRUN' + Ext;
  CpF[8] := 'DRRUN.DFM';
  CpF[9] := 'DRRUNDEF' + Ext;
  CpF[10] := 'DRRUNDEF.DFM';
  CpF[11] := 'DRSETDTA' + Ext;
  CpF[12] := 'DRSETDTA.DFM';
  CpF[13] := 'DRUTILS' + Ext;
  If rg16or32.ItemIndex = 0 Then
  Begin
    CpF[14] := 'DYNA16.HLP';
    CpF[15] := 'DYNA16.DCR';
    CpF[21] := 'DRAS16.DCR';
  End Else
  Begin
    CpF[14] := 'DYNA32.HLP';
    CpF[15] := 'DYNA32.DCR';
    CpF[21] := 'DRAS32.DCR';
  End;
{$IFNDEF Shareware}
  { DYNAxx.DCR --> DYNAREQS.DCR }
  If Not CopyFile(Src + CpF[15], Src + 'DYNAREQS.DCR', Err) Then
      Raise Exception.Create(Err);
  If Not CopyFile(Src + CpF[21], Src + 'DRASSIST.DCR', Err) Then
      Raise Exception.Create(Err);
{$ENDIF}
  CpF[15] := 'DYNAREQS.DCR';
  CpF[16] := 'DYNAREQS' + Ext;
  CpF[17] := 'README.TXT';
  CpF[18] := 'DYNAREQS.RGT';
  CpF[19] := 'ORDER.TXT';
  CpF[20] := 'DRASSIST' + Ext;
  CpF[21] := 'DRASSIST.DCR';
  { Examples }
  ExF[1] := 'CITY.DB';
  ExF[2] := 'CITY.PX';
  ExF[3] := 'SELECTED.DB';
  ExF[4] := 'SELECTED.PX';
  ExF[5] := 'SELECTED.VAL';
  ExF[6] := 'CUSTLANG.DB';
  ExF[7] := 'CUSTOMER.DB';
  ExF[8] := 'CUSTOMER.PX';
  ExF[9] := 'QUERYDEF.DB';
  ExF[10] := 'QUERYDEF.MB';
  ExF[11] := 'QUERYDEF.PX';
  ExF[12] := 'EXAMPLE.DFM';
  ExF[13] := 'EXAMPLE.PAS';
  ExF[14] := 'EXAMPLE.TXT';
  ExF[15] := 'LANGUAGE.DB';
  ExF[16] := 'LANGUAGE.PX';
  ExF[17] := 'SPECIALT.DB';
  ExF[18] := 'SPECIALT.PX';

  { Copy Component Files }
  For I := 1 To CpNb Do FicCopy(CpF[I]);
  { Copy Example Files }
  Dst := DestEx + '\';
  For I := 1 To ExNb Do FicCopy(ExF[I]);

  Indique('Purging Files');
  If Source <> Destination Then
    { Purge Component Files }
    For I := 1 To CpNb Do FicDelete(CpF[I]);
  If Source <> DestEx Then
    { Purge Example Files }
    For I := 1 To ExNb Do FicDelete(ExF[I]);
End;

Function TfmInstall.Run: Boolean;
Var
  IFile : TIniFile;
  Ok : Boolean;
  I : Integer;
Begin
  OpEc := 0;
  Indique('Verifying Parameters');
  SetIndicators(True);
  Try
    If Dispo < Needed Then
      Raise Exception.Create('Insuffisant Disk Space');
    If rg16or32.ItemIndex = -1 Then
      Raise Exception.Create('Delphi 16 or 32 bits ? Take a Decision');

    Destination := edRepert.Text;
    DestEx := edRepertExamples.Text;
    Ok := False;
    { ***** Create DynaReqs.INI ****** }
    Indique('Creating ' + Destination + '\DYNAREQS.INI');
    Try
      IFile := TIniFile.Create(Destination + '\DYNAREQS.INI');
      IFile.WriteString('Current Directory', 'Install', 'Ok');
    Finally
      IFile.Free;
    End;

    CopyFiles;      { ***** Copy files ****** }
    { ***** Create Section in WIN.INI ****** }
    Try
      IFile := TIniFile.Create('WIN.INI');
      IFile.WriteString('TDynamicRequester', 'Path', Destination);
      IFile.WriteString('TDynamicRequester', 'Company', edSociete.Text);
      IFile.WriteString('TDynamicRequester', 'User', edNom.Text);
      IFile.WriteString('TDynamicRequester', 'Registration', edRegistrationID.Text);
    Finally
      IFile.Free;
    End;
    NoteBook1.ActivePage := 'Bye';
    Ok := True;
  Finally
    SetIndicators(False);
    Result := Ok;
  End;
End;

procedure TfmInstall.btInstallClick(Sender: TObject);
Begin
  Run;
End;

procedure TfmInstall.btOkClick(Sender: TObject);
begin
  Close;
end;

procedure TfmInstall.btChooseRepertSourceClick(Sender: TObject);
begin
  Source := edSource.Text;
  If SelectDirectory(Source, [], 0) Then
    edSource.Text := Source;
end;

procedure TfmInstall.btChooseRepertClick(Sender: TObject);
var
  Dir: string;
begin
  If Sender = btChooseRepert Then Dir := edRepert.Text
                             Else Dir := edRepertExamples.Text;
  If SelectDirectory(Dir, [sdAllowCreate, sdPerformCreate, sdPrompt], 0) Then
    If Sender = btChooseRepert Then edRepert.Text := Dir
                               Else edRepertExamples.Text := Dir;
end;

procedure TfmInstall.edRepertChange(Sender: TObject);
Var
  Lect : Byte;
begin
  Lect := Ord(UpCase(edRepert.Text[1]))-64;
  Dispo := DiskFree(Lect);
  If Dispo < Needed Then edFree.Font.Color := clRed
                    Else edFree.Font.Color := clBlack;
  btInstall.Enabled := Dispo >= Needed;
  edFree.Text := StrEspace(Dispo);
end;

procedure TfmInstall.Notebook1PageChanged(Sender: TObject);
begin
  Case Notebook1.PageIndex Of
    0 : Begin
          btPrevious.Enabled := False;
          btNext.Enabled := True;
        End;
    1 : Begin
          btPrevious.Enabled := True;
          btNext.Enabled := True;
          btInstall.Visible := False;
        End;
    2 : Begin
          btPrevious.Enabled := True;
          btNext.Enabled := False;
          btInstall.Visible := True;
        End;
    3 : Begin
          btPrevious.Visible := False;
          btNext.Visible := False;
          btInstall.Visible := False;
          btCancel.Visible := False;
          btOK.Visible := True;
        End;
  End;
end;

{ Utilities }
Procedure drExcept(Const Mess : String);
Begin
  MessageDlg(Mess, mtError, [mbOk], 0);
  Abort;
End;

Function YesNo(Const Mess : String) : Boolean;
Begin
  Result := MessageDlg(Mess, mtWarning, [mbYes, mbNo], 0) = mrYes;
End;

Procedure SetOpenError(ErrCode: Word; S: String; Var pErr: String);
Begin
  Case ErrCode Of
    $0001: pErr := 'Invalid function';
    $0002: pErr := 'File not found';
    $0003: pErr := 'Path not found';
    $0004: pErr := 'Too many open files';
    $0005: pErr := 'Access denied';
    $0006: pErr := 'Invalid handle';
    $0007: pErr := 'Arena trashed';
    $0008: pErr := 'Not enough memory';
    $0009: pErr := 'Invalid block';
    $000A: pErr := 'Bad environment';
    $000B: pErr := 'Bad format';
    $000C: pErr := 'Invalid access';
    $000D: pErr := 'Invalid data';
    $000F: pErr := 'Invalid drive';
    $0010: pErr := 'Current directory';
    $0011: pErr := 'Not same device';
    $0012: pErr := 'No more files';
    $0013: pErr := 'Write protect error';
    $0014: pErr := 'Bad unit';
    $0015: pErr := 'Not ready';
    $0016: pErr := 'Bad command';
    $0017: pErr := 'CRC error';
    $0018: pErr := 'Bad length';
    $0019: pErr := 'Seek error';
    $001A: pErr := 'Not MS-DOS disk';
    $001B: pErr := 'Sector not found';
    $001C: pErr := 'Out of paper';
    $001D: pErr := 'Write fault';
    $001E: pErr := 'Read fault';
    $001F: pErr := 'General failure';
    $0020: pErr := 'Sharing violation';
    $0021: pErr := 'Lock violation';
    $0022: pErr := 'Wrong disk';
    $0023: pErr := 'File control block unavailable';
    $0024: pErr := 'Sharing buffer exceeded';
    $0032: pErr := 'Not supported';
    $0033: pErr := 'Remote not listed';
    $0034: pErr := 'Duplicate name';
    $0035: pErr := 'Bad netpath';
    $0036: pErr := 'Network busy';
    $0037: pErr := 'Device does not exist';
    $0038: pErr := 'Too many commands';
    $0039: pErr := 'Adaptor hardware error';
    $003A: pErr := 'Bad network response';
    $003B: pErr := 'Unexpected network error';
    $003C: pErr := 'Bad remote adaptor';
    $003D: pErr := 'Print queue full';
    $003E: pErr := 'No spool space';
    $003F: pErr := 'Print canceled';
    $0040: pErr := 'Netname deleted';
    $0041: pErr := 'Network access denied';
    $0042: pErr := 'Bad device type';
    $0043: pErr := 'Bad network name';
    $0044: pErr := 'Too many names';
    $0045: pErr := 'Too many sessions';
    $0046: pErr := 'Sharing paused';
    $0047: pErr := 'Request not accepted';
    $0048: pErr := 'Redirection paused';
    $0050: pErr := 'File exists';
    $0051: pErr := 'Duplicate file control block';
    $0052: pErr := 'Cannot make';
    $0053: pErr := 'Interrupt 24 failure';
    $0054: pErr := 'Out of structures';
    $0055: pErr := 'Already assigned';
    $0056: pErr := 'Invalid password';
    $0057: pErr := 'Invalid parameter';
    $0058: pErr := 'Net write fault';
  End;
  pErr := 'Error during LZOpenFile(' + S + '): ' + pErr;
  Abort;
End;

Procedure SetCopyError(ErrCode: LongInt; S: String; Var pErr: String);
Begin
  Case ErrCode Of
    LZERROR_BADINHANDLE: pErr := 'Invalid Source File Handle';
    LZERROR_BADOUTHANDLE: pErr:= 'Invalid Destination File Handle';
    LZERROR_BADVALUE: pErr := 'Bad Input Parameter';
    LZERROR_GLOBALLOC: pErr := 'Insufficient Memory';
    LZERROR_GLOBLOCK: pErr := 'Invalid Internal Data Structure';
    LZERROR_READ: pErr := 'Invalid Source File Format';
    LZERROR_UNKNOWNALG : pErr := 'Unrecognized Compression Algorithm';
    LZERROR_WRITE : pErr := 'Insufficient Space for Output File';
  End;
  pErr := 'Error during LZCopy(' + S + '): ' + pErr;
  Abort;
End;

Function DeleteFile(pFile : String; Var pErr: String): Boolean;
Var
  R : Boolean;
  F : PChar;
  FHandle : Integer;
  ReOpenBuf: TOFStruct;
  L : LongInt;
begin
  R := False;
  pErr := '';
  Try
    F := StrAlloc(Length(pFile) + 1); StrPCopy(F, pFile);

    FHandle := LZOpenFile(F, ReOpenBuf, OF_DELETE);
    If FHandle = -1 Then SetOpenError(ReOpenBuf.nErrCode, pFile, pErr);
    LZClose(FHandle);
    R := True;
  Except
  End;
  StrDispose(F);
  Result := R;
end;

Function CopyFile(pSource, pDest : String; Var pErr: String): Boolean;
Var
  R : Boolean;
  S, D : PChar;
  SHandle, DHandle : Integer;
  ReOpenBuf: TOFStruct;
  L : LongInt;
begin
  R := False;
  pErr := '';
  Try
    S := StrAlloc(Length(pSource) + 1); StrPCopy(S, pSource);
    D := StrAlloc(Length(pDest) + 1);   StrPCopy(D, pDest);

    SHandle := LZOpenFile(S, ReOpenBuf, OF_READ);
    If SHandle = -1 Then SetOpenError(ReOpenBuf.nErrCode, pSource, pErr);
    DHandle := LZOpenFile(D, ReOpenBuf, OF_CREATE);
    If DHandle = -1 Then SetOpenError(ReOpenBuf.nErrCode, pDest, pErr);
    L := LZCopy(SHandle, DHandle);
    If L < 0 Then SetCopyError(L, pSource, pErr);
    LZClose(SHandle); LZClose(DHandle);
    R := True;
  Except
  End;
  StrDispose(S);
  StrDispose(D);
  Result := R;
end;

Function CreateFile(pDest : String; Var pErr: String): Boolean;
Var
  R : Boolean;
  D : PChar;
  DHandle : Integer;
  ReOpenBuf: TOFStruct;
  L : LongInt;
begin
  R := False;
  pErr := '';
  Try
    D := StrAlloc(Length(pDest) + 1);   StrPCopy(D, pDest);
    DHandle := LZOpenFile(D, ReOpenBuf, OF_CREATE);
    If DHandle = -1 Then SetOpenError(ReOpenBuf.nErrCode, pDest, pErr);
    LZClose(DHandle);
    R := True;
  Except
  End;
  StrDispose(D);
  Result := R;
end;

End.
